import clilib.util

logger = clilib.util.get_logger(f"[{__name__}]")


def verb(*args, **kwargs):
    logger.debug('Decorating verb')
    logger.debug(f"args  : {args}")
    logger.debug(f"kwargs: {kwargs}")

    func = args[0]
    logger.debug(f"func  : {func}")

    action = func.__name__
    logger.debug(f"action: {action}")
    logger.debug(f"Adding attribute '_action={action}' to decorated method")
    setattr(func, '_action', action)

    return func



import clilib.util
import clilib.decorator.util


logger = clilib.util.get_logger(f"[{__name__}]")


def resource(klass):
    logger.debug(f"class: {klass}")

    logger.debug("Adding attributes '_parsers' and '_name'")
    setattr(klass, '_parsers', {})
    setattr(klass, '_name', '')

    logger.debug(f"Finding verb decorated functions in {klass}")
    for key, value in klass.__dict__.items():
        logger.debug(f"Checking {key}")
        if callable(value) and clilib.decorator.util.is_verb(value):
            logger.debug(f"{key} is decorated!")
            clilib.register_verb(klass, value)

            if hasattr(value, '_args'):
                logger.debug('Adding args')

    return klass



import clilib.util
import clilib.decorator as decorator
import argparse
import logging


_root_parser = argparse.ArgumentParser()
_subparsers = _root_parser.add_subparsers(dest='_cmd')
_subparsers.required = True
_args = None

logger = clilib.util.get_logger(f"[{__name__}]")


def build_parser(parser):
    return parser


def register_verb(resource, func):
    logger.debug('Registering resource verb')
    logger.debug(_subparsers.choices)

    resource_name = clilib.util.to_kebab(resource.__name__)
    verb = getattr(func, '_action')
    args = getattr(func, '_args', []) + getattr(resource, '_args', [])

    logger.debug(f"resource: {resource}")
    logger.debug(f"func    : {func}")
    logger.debug(f"verb    : {verb}")
    logger.debug(f"args    : {args}")

    if verb not in _subparsers.choices:
        logger.debug(f"Adding verb, '{verb}', in _subparsers")
        _subparsers.add_parser(verb)

    if verb not in resource._parsers:
        logger.debug(f"Adding verb, '{verb}', in resource._parsers")
        if verb in _subparsers.choices:
            logger.debug(f"Referencing subparser of _subparsers.choices['{verb}']")
            resource._parsers[verb] = clilib.util.get_subparser(_subparsers.choices[verb])
        else:
            logger.debug(f"Creating _subparsers.choices['{verb}']")
            resource._parsers[verb] = _subparsers.choices[verb].add_subparsers()

    logger.debug(resource._parsers[verb].choices)
    if resource_name not in resource._parsers[verb].choices:
        logger.debug(f"Adding resource, {resource}, target for verb, {verb}, as {resource_name}")
        resource_parser = resource._parsers[verb].add_parser(resource_name)
        resource_parser.set_defaults(_func=func, _klass=resource)

        for arg in args:
            rargs, rkwargs = arg

            logger.debug(f"Adding arg to '{verb} {resource_name}': {rargs}, {rkwargs}")
            resource_parser.add_argument(*rargs, **rkwargs)

# Primarily for testing
# Future: Reading from config file or something
def init(prog):
    global _root_parser

    _root_parser.prog = prog


def run(prog):
    global _args
    global _subparsers

    _args = _root_parser.parse_args()

    if hasattr(_args, '_func'):
        _args._func(_args._klass, args=_args)



import clilib.util


logger = clilib.util.get_logger(f"[{__name__}]")


def arg(*decorator_args, **decorator_kwargs):
    logger.debug('Handling @arg decorator')
    logger.debug(f"args  : {decorator_args}")
    logger.debug(f"kwargs: {decorator_kwargs}")

    def wrapper(func):
        logger.debug(f"@arg is wrapping: {func}")

        if not hasattr(func, '_args'):
            logger.debug('_args attribute not found, setting as []')
            setattr(func, '_args', [])

        logger.debug('Retrieving _args attribute')
        _args = getattr(func, '_args')

        logger.debug(f"Appending to _args: ({decorator_args}, {decorator_kwargs})")
        _args.append((decorator_args, decorator_kwargs))

        return func

    return wrapper



import re
import logging

import argparse
from pylogrus import PyLogrus, TextFormatter

_log_level = logging.INFO


# https://stackoverflow.com/questions/1175208/elegant-python-function-to-convert-camelcase-to-snake-case
def to_kebab(word):
    word = re.sub('(.)([A-Z][a-z]+)', r'\1-\2', word)
    return re.sub('([a-z0-9])([A-Z])', r'\1-\2', word).lower()


def get_logger(name=__name__):
    logging.setLoggerClass(PyLogrus)
    logger = logging.getLogger(name)
    logger.setLevel(_log_level)

    formatter = TextFormatter(datefmt='Z', colorize=True)

    ch = logging.StreamHandler()
    ch.setLevel(_log_level)
    ch.setFormatter(formatter)
    logger.addHandler(ch)

    return logger.withPrefix(name)


def get_subparser(parser):
    # Couldn't find a function in argparse to do this
    # It's already stored in the parser object. Doesn't make sense to have another mapping for it
    for action in parser.__dict__['_actions']:
        if (type(action) is argparse._SubParsersAction):
            return action

    subparser = parser.add_subparsers(dest='_cmd')
    subparser.required = True

    return subparser



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="pyclilib",
    version="0.3.1",
    author="Mark Diez",
    author_email="markediez@gmail.com",
    description="Build CLI Tools with ease",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/markediez/clilib",
    packages=setuptools.find_packages('src'),
    package_dir={"": "src"},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        'pylogrus'
    ]
)
